@@ -42,9 +42,6 @@ namespace PlasmaQuick
 
 QHash<QObject *, AppletQuickItem *> AppletQuickItemPrivate::s_rootObjects = QHash<QObject *, AppletQuickItem *>();
 
-//TODO: temporary
-QSet<QString> AppletQuickItemPrivate::s_legacyApplets = QSet<QString>({QStringLiteral("org.kde.plasma.bluetooth"), QStringLiteral("org.kde.plasma.pager"), QStringLiteral("org.kde.desktopcontainment"), QStringLiteral("org.kde.plasma.folder"), QStringLiteral("org.kde.panel"), QStringLiteral("org.kde.plasma.analogclock"), QStringLiteral("org.kde.plasma.battery"), QStringLiteral("org.kde.plasma.notifications"), QStringLiteral("org.kde.plasma.systemtray")});
-
 
 AppletQuickItemPrivate::AppletQuickItemPrivate(Plasma::Applet *a, AppletQuickItem *item)
     : q(item),
@@ -53,27 +50,20 @@ AppletQuickItemPrivate::AppletQuickItemPrivate(Plasma::Applet *a, AppletQuickIte
       applet(a),
       expanded(false)
 {
-    if (!a->pluginInfo().isValid()) {
+}
+
+void AppletQuickItemPrivate::init()
+{
+    if (!applet->pluginInfo().isValid()) {
         // This `qmlObject` is used in other parts of the code
         qmlObject = new KDeclarative::QmlObject(q);
         return;
     }
 
-    //TODO: remove the legacy support at some point
-    //use the shared engine only for applets that are nt in the legacy list
-    //if they are, use the shared engine if their mayor version is at least 3
-    const QStringList version = a->pluginInfo().version().split(".");
-    if (!AppletQuickItemPrivate::s_legacyApplets.contains(a->pluginInfo().pluginName()) ||
-         (!version.isEmpty() && version.first().toInt() >= 3)) {
-
-        qmlObject = new KDeclarative::QmlObjectSharedEngine(q);
-        if (!qmlObject->engine()->urlInterceptor()) {
-            PackageUrlInterceptor *interceptor = new PackageUrlInterceptor(qmlObject->engine(), Plasma::Package());
-            qmlObject->engine()->setUrlInterceptor(interceptor);
-        }
-    } else {
-        qWarning() << "Falling back to legacy separed QQmlEngine for applet" << a->pluginInfo().pluginName();
-        qmlObject = new KDeclarative::QmlObject(q);
+    qmlObject = new KDeclarative::QmlObjectSharedEngine(q);
+    if (!qmlObject->engine()->urlInterceptor()) {
+        PackageUrlInterceptor *interceptor = new PackageUrlInterceptor(qmlObject->engine(), Plasma::Package());
+        qmlObject->engine()->setUrlInterceptor(interceptor);
     }
 }
 
@@ -255,8 +245,8 @@ void AppletQuickItemPrivate::compactRepresentationCheck()
         return;
     }
 
-    //ignore 0,0 sizes;
-    if (q->width() <= 0 && q->height() <= 0) {
+    //ignore 0 sizes;
+    if (q->width() <= 0 || q->height() <= 0) {
         return;
     }
 
@@ -397,29 +387,35 @@ AppletQuickItem::AppletQuickItem(Plasma::Applet *applet, QQuickItem *parent)
     : QQuickItem(parent),
       d(new AppletQuickItemPrivate(applet, this))
 {
+    d->init();
+
     if (d->applet) {
         d->appletPackage = d->applet->package();
-    }
-    if (d->applet && d->applet->containment() && d->applet->containment()->corona()) {
-        d->coronaPackage = d->applet->containment()->corona()->package();
-    }
-    if (d->applet && d->applet->containment()) {
-        d->containmentPackage = d->applet->containment()->package();
+
+        if (d->applet->containment()) {
+            if (d->applet->containment()->corona()) {
+                d->coronaPackage = d->applet->containment()->corona()->package();
+            }
+
+            d->containmentPackage = d->applet->containment()->package();
+        }
+
+        if (d->applet->pluginInfo().isValid()) {
+            const QString rootPath = d->applet->pluginInfo().property(QStringLiteral("X-Plasma-RootPath")).toString();
+            if (!rootPath.isEmpty()) {
+                d->qmlObject->setTranslationDomain(QLatin1String("plasma_applet_") + rootPath);
+            } else {
+                d->qmlObject->setTranslationDomain(QLatin1String("plasma_applet_") + d->applet->pluginInfo().pluginName());
+            }
+        }
+
+        // set the graphicObject dynamic property on applet
+        d->applet->setProperty("_plasma_graphicObject", QVariant::fromValue(this));
     }
 
-    if (applet->pluginInfo().isValid()) {
-        const QString rootPath = applet->pluginInfo().property(QStringLiteral("X-Plasma-RootPath")).toString();
-        if (!rootPath.isEmpty()) {
-            d->qmlObject->setTranslationDomain(QLatin1String("plasma_applet_") + rootPath);
-        } else {
-            d->qmlObject->setTranslationDomain(QLatin1String("plasma_applet_") + applet->pluginInfo().pluginName());
-        }
-    }
     d->qmlObject->setInitializationDelayed(true);
 
-    // set the graphicObject dynamic property on applet
-    d->applet->setProperty("_plasma_graphicObject", QVariant::fromValue(this));
-    setProperty("_plasma_applet", QVariant::fromValue(applet));
+    setProperty("_plasma_applet", QVariant::fromValue(d->applet));
 }
 
 AppletQuickItem::~AppletQuickItem()
@@ -535,16 +531,19 @@ void AppletQuickItem::init()
     QVariantHash initialProperties;
     //initialize with our size only if valid
     if (width() > 0 && height() > 0) {
-        initialProperties[QStringLiteral("width")] = width();
-        initialProperties[QStringLiteral("height")] = height();
+        const qreal w = parentItem() ? std::min(parentItem()->width(), width()) : width();
+        const qreal h = parentItem() ? std::min(parentItem()->height(), height()) : height();
+        initialProperties[QStringLiteral("width")] = w;
+        initialProperties[QStringLiteral("height")] = h;
     }
     d->qmlObject->setInitializationDelayed(false);
     d->qmlObject->completeInitialization(initialProperties);
 
     //otherwise, initialize our size to root object's size
     if (d->qmlObject->rootObject() && (width() <= 0 || height() <= 0)) {
-        setWidth(d->qmlObject->rootObject()->property("width").value<qreal>());
-        setHeight(d->qmlObject->rootObject()->property("height").value<qreal>());
+        const qreal w = d->qmlObject->rootObject()->property("width").value<qreal>();
+        const qreal h = d->qmlObject->rootObject()->property("height").value<qreal>();
+        setSize(parentItem() ? QSizeF(std::min(parentItem()->width(), w), std::min(parentItem()->height(), h)) : QSizeF(w, h));
     }
 
     //default fullrepresentation is our root main component, if none specified
@@ -574,9 +573,6 @@ void AppletQuickItem::init()
         d->compactRepresentationExpander->loadUrl(QUrl::fromLocalFile(compactExpanderPath));
     }
 
-    //HACK: check the Layout properties we wrote
-    QQmlProperty p(this, "Layout.minimumWidth", QtQml::qmlContext(d->qmlObject->rootObject()));
-
     d->compactRepresentationCheck();
     qmlObject()->engine()->rootContext()->setBaseUrl(qmlObject()->source());
     qmlObject()->engine()->setContextForObject(this, qmlObject()->engine()->rootContext());
@@ -653,6 +649,21 @@ QQmlComponent *AppletQuickItem::fullRepresentation()
     return d->fullRepresentation;
 }
 
+QObject *AppletQuickItem::testItem()
+{
+    if (!d->testItem) {
+        const QUrl url = QUrl::fromLocalFile(d->appletPackage.filePath("test"));
+        if (url.isEmpty()) {
+            return nullptr;
+        }
+
+        d->testItem = d->qmlObject->createObjectFromSource(url, QtQml::qmlContext(rootItem()));
+        d->testItem->setProperty("plasmoidItem", QVariant::fromValue<QObject*>(this));
+    }
+
+    return d->testItem;
+}
+
 void AppletQuickItem::setFullRepresentation(QQmlComponent *component)
 {
     if (d->fullRepresentation == component) {
@@ -736,6 +747,21 @@ QObject *AppletQuickItem::rootItem()
     return d->qmlObject->rootObject();
 }
 
+void AppletQuickItem::childEvent(QChildEvent *event)
+{
+    // Added child may be QQuickLayoutAttached
+    if (event->added() && !d->ownLayout && d->currentRepresentationItem) {
+        // Child has not yet finished initialization at this point
+        QTimer::singleShot(0, this, [this]() {
+            if (!d->ownLayout) {
+                d->connectLayoutAttached(d->currentRepresentationItem);
+            }
+        });
+    }
+
+    QQuickItem::childEvent(event);
+}
+
 void AppletQuickItem::geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry)
 {
     Q_UNUSED(oldGeometry)
