@@ -24,7 +24,7 @@
 #include <QQmlListProperty>
 #include <QAbstractListModel>
 
-#include <plasmaquick_export.h>
+#include <plasmaquick/plasmaquick_export.h>
 
 //
 //  W A R N I N G
